describe('node-openjtalk', function() {
  var fs = require('fs');
  var OpenJTalk = require('..').OpenJTalk;
  var Speaker = require('speaker');
  var riffPrinter = require('./riff-printer');


  it('instantiate OpenJTalk', function() {
    var fn_voice = OpenJTalk.voices.m001;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
  });

  it('OpenJTalk#synthesizeSync', function(done) {
    this.timeout(5000);
    var fd = fs.openSync(__dirname + '/buffer.wav', 'w');
    var speaker = new Speaker({
      channels: 1,
      bitDepth: 16,
      sampleRate: 48000
    });
    var fn_voice = OpenJTalk.voices.mei_normal;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "すもももももももものうち";
    var buffer = open_jtalk.synthesizeSync(text);
    riffPrinter(buffer);
    speaker.end(buffer);
    speaker.on('close', function() {
      done();
    });
    // fs.write(fd, buffer, 0, buffer.length, null, function(err, written, buffer) {
    //   fs.closeSync(fd);
    //   if (err) {
    //     throw err;
    //   }
    //   done();
    // });
  });

  it('can synthesize multiple times', function() {
    var fn_voice = OpenJTalk.voices.m001;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "すもももももももものうち";
    open_jtalk.synthesizeSync(text);
    open_jtalk.synthesizeSync(text);
  });

  it('can synthesize multiple times asynchronously', function(done) {
    var fn_voice = OpenJTalk.voices.m001;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "すもももももももものうち";
    var finished = 0;
    open_jtalk.synthesize(text, function(error, buffer) {
      console.log('error', error);
      if (error) {
        throw error;
      }
      if (++finished === 2) {
        done();
      }
    });
    open_jtalk.synthesize("隣の客はよく柿食う客だ", function(error, buffer) {
      console.log(buffer);
      if (error) {
        throw error;
      }
      if (++finished === 2) {
        done();
      }
    });
  });

  it('can change parameters and synthesize multiple times asynchronously', function(done) {
    var fn_voice = OpenJTalk.voices.m001;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "すもももももももものうち";
    var finished = 0;
    open_jtalk.synthesize(text, function(option) {
      option.speed += 1;
      return option;
    }, function(error, buffer) {
      if (error) {
        throw error;
      }
      if (++finished === 2) {
        done();
      }
    });
    open_jtalk.synthesize("隣の客はよく柿食う客だ", function(option) {
      option.speed += 1;
      return option;
    }, function(error, buffer) {
      if (error) {
        throw error;
      }
      if (++finished === 2) {
        done();
      }
    });
  });

  it('can synthesize asynchronously large text', function(done) {
    this.timeout(10000);
    var fd = fs.openSync(__dirname + '/async.wav', 'w');
    var fn_voice = OpenJTalk.voices.mei_normal;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "吾輩わがはいは猫である。名前はまだ無い。どこで生れたかとんと見当けんとうがつかぬ。何でも薄暗いじめじめした所でニャーニャー泣いていた事だけは記憶している。吾輩はここで始めて人間というものを見た。";
    open_jtalk.synthesize(text, function(error, buffer) {
      riffPrinter(buffer);
      var speaker = new Speaker({
        channels: 1,
        bitDepth: 16,
        sampleRate: 48000
      });
      //speaker.end(buffer);
      speaker.on('close', function() {
        done();
      });
      if (error) {
        throw error;
      }
      // fs.write(fd, buffer, 0, buffer.length, null, function(err, written, buffer) {
      //   fs.closeSync(fd);
      //   if (err) {
      //     throw err;
      //   }
      //   done();
      // });
    });
  });

  it('change speed', function(done) {
    var fd = fs.openSync(__dirname + '/speed.wav', 'w');
    var fn_voice = OpenJTalk.voices.mei_normal;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var text = "すもももももももものうち";
    open_jtalk.synthesize(text, function(option) {
      option.speed -= 0.2;
      option.addHalfTone = 0.1;
      option.beta += 0.3;
      return option;
    }, function(error, buffer) {
      if (error) {
        throw error;
      }
      fs.write(fd, buffer, 0, buffer.length, null, function(err, written, buffer) {
        fs.closeSync(fd);
        if (err) {
          throw err;
        }
        done();
      });
    });
  });

  it('change tone', function() {
    var fd = fs.openSync(__dirname + '/tone.wav', 'w');
    var fn_voice = OpenJTalk.voices.m001;
    var open_jtalk = new OpenJTalk({voice: fn_voice});
    var buf = "隣の客はよく柿食う客だ";
    var buffer = open_jtalk.synthesizeSync(buf, function(option) {
      return {
        addHalfTone: option.addHalfTone + 1
      };
    });
    fs.write(fd, buffer, 0, buffer.length, null, function(err, written, buffer) {
      fs.closeSync(fd);
      if (err) {
        throw err;
      }
      done();
    });
  });
});
